package com.gic.enterprise.service.outer.impl.union;

import com.gic.api.base.commons.Page;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResOperateEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthStatusEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.union.*;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthApp;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthResDetail;
import com.gic.enterprise.qo.union.UnionEnterpriseAuthResLogQO;
import com.gic.enterprise.service.*;
import com.gic.store.service.StoreAuthorizationApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.validation.annotation.Validated;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 1:52 PM 
 */
@Service("unionEnterpriseAuthApiService")
public class UnionEnterpriseAuthApiServiceImpl implements UnionEnterpriseAuthApiService{
    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private UnionEnterpriseAuthResDetailService unionEnterpriseAuthResDetailService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private UnionEnterpriseAuthAppService unionEnterpriseAuthAppService;
    @Autowired
    private UnionEnterpriseAuthResLogService unionEnterpriseAuthResLogService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;

    @Override
    public ServiceResponse<Void> saveUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        Assert.notNull(ownEnterpriseId, "自有商户不能为空");
        Assert.notNull(unionEnterpriseId, "联合商户不能为空");
        if (ownEnterpriseId.equals(unionEnterpriseId)) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "不能联合自有商户");
        }
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old != null) {
            return ServiceResponse.success();
        }
        unionEnterpriseAuthService.saveUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> authResource(@Validated UnionEnterpriseAuthResDTO dto) {
        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(dto.getOwnEnterpriseId(), dto.getUnionEnterpriseId());
        //是否是编辑。编辑和新增授权同步资源处理不同
        boolean isEdit = old.getResourceGroupId() != null;
        //todo starrynight 通知业务方，同步数据
        Integer resourceGroupId = dto.getResourceGroupId();
        ServiceResponse<ResourceGroupDTO> resourceResponse = resourceGroupApiService.getResourceGroup(resourceGroupId);
        if (!resourceResponse.isSuccess() || resourceResponse.getResult() == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组数据不存在");
        }
        //
        checkResourceDealing((old.getUnionEnterpriseAuthId()));
        if (isEdit) {

            //todo 重置最新联合时间和解除时间？
        } else {

        }
        ResourceGroupDTO resourceGroupDTO = resourceResponse.getResult();
        //门店通知
        if (resourceGroupDTO.getStoreResource() != null) {
            storeAuthorizationApiService.authStore(resourceGroupDTO.getStoreResource().intValue(), dto.getUnionEnterpriseId(), dto.getOwnEnterpriseId());
        }

        UnionEnterpriseAuthDTO data = EntityUtil.changeEntityNew(UnionEnterpriseAuthDTO.class, dto);
        data.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
        unionEnterpriseAuthService.update(data);

        //初始化未授权资源数据
        unionEnterpriseAuthResDetailService.initUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UnionEnterpriseAuthDTO> getUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UnionEnterpriseAuthDTO.class, old));
    }

    @Override
    public ServiceResponse<Void> resetAuthResource(Integer ownEnterpriseId, Integer unionEnterpriseId, Integer resourceType) {
        Assert.notNull(ownEnterpriseId, "自有商户不能为空");
        Assert.notNull(unionEnterpriseId, "联合商户不能为空");
        Assert.notNull(resourceType, "资源类型不能为空");

        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);

        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), resourceType);
        if (detail == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源不存在");
        }
        //todo 重新授权
        //门店资源
        String authKey = detail.getAuthKey();
        storeAuthorizationApiService.reAuthStore(authKey);

        //日志
        unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(),
                resourceType, UnionEnterpriseAuthResOperateEnum.RE_AUTH.getMessage());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveUnion(Integer ownEnterpriseId, Integer unionEnterpriseId, String relieveReason) {
        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);

        UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
        dto.setRelieveTime(new Date());
        //解除生效时间，保留X个自然日后开始解除资源
        dto.setRelieveEffectDate(7);
        dto.setRelieveRemark(relieveReason);
        dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
        dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.TO_DO_RELIEVE.getCode());
        unionEnterpriseAuthService.update(dto);

        //资源状态
        unionEnterpriseAuthResDetailService.relieveUnion(old.getUnionEnterpriseAuthId());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthListDTO>> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, String enterpriseName, Integer authStatus) {
        //联合商户名称匹配范围
        List<Integer> unionEnterpriseIdList = new ArrayList<>();
        if (StringUtils.isNotBlank(enterpriseName)) {
            List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterpriseByName(enterpriseName);
            if (CollectionUtils.isNotEmpty(enterpriseList)) {
                unionEnterpriseIdList = enterpriseList
                        .stream()
                        .filter(e -> !e.getEnterpriseId().equals(ownEnterpriseId))
                        .mapToInt(e -> e.getEnterpriseId())
                        .boxed()
                        .collect(Collectors.toList());
            }
        }

        List<UnionEnterpriseAuthListDTO> result = unionEnterpriseAuthService.listUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseIdList, authStatus);
        if (CollectionUtils.isNotEmpty(result)) {
            //组装数据
            //1:联合商户数据
            List<Integer> enterpriseIdList = result.stream().mapToInt(e -> e.getEnterpriseId()).boxed().collect(Collectors.toList());
            Map<Integer, TabEnterprise> enterpriseMap = getEnterprise(enterpriseIdList);

            //2:超管数据
            Map<Integer, UserDTO> userMap = getUser(enterpriseIdList);

            //3:资源明细
            List<Integer> unionEnterpriseAuthIdList = result.stream().mapToInt(e -> e.getUnionEnterpriseAuthId()).boxed().collect(Collectors.toList());
            List<TabSysUnionEnterpriseAuthResDetail> list = unionEnterpriseAuthResDetailService.listByUnionEnterpriseAuthIdList(unionEnterpriseAuthIdList);
            Map<Integer, List<TabSysUnionEnterpriseAuthResDetail>> unionEnterpriseAuthResMap = list
                    .stream()
                    .collect(Collectors.groupingBy(TabSysUnionEnterpriseAuthResDetail::getUnionEnterpriseAuthId));

            //填充数据
            for (UnionEnterpriseAuthListDTO obj : result) {
                TabEnterprise enterprise = enterpriseMap.get(obj.getEnterpriseId());
                if (enterprise != null) {
                    obj.setCompanyName(enterprise.getCompanyName());
                    obj.setAddress(enterprise.getAddress());
                    obj.setEnterpriseName(enterprise.getEnterpriseName());
                }
                UserDTO user = userMap.get(obj.getEnterpriseId());
                if (user != null) {
                    obj.setUserName(user.getUserName());
                    obj.setPhoneAreaCode(user.getPhoneAreaCode());
                    obj.setPhoneNumber(user.getPhoneNumber());
                }
                List<TabSysUnionEnterpriseAuthResDetail> tempDetail = unionEnterpriseAuthResMap.get(obj.getUnionEnterpriseAuthId());
                obj.setUnionEnterpriseResourceList(EntityUtil.changeEntityListNew(UnionEnterpriseAuthResDetailDTO.class, tempDetail));
            }
            return ServiceResponse.success(result);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthDTO>> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listUnionEnterpriseAuth(ownEnterpriseId);
        if (list == null) {
            return ServiceResponse.success(Collections.emptyList());
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(UnionEnterpriseAuthDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> saveOrUpdateUnionEnterpriseAuthApp(@Valid UnionEnterpriseAuthAppDTO dto) {
        Integer unionEnterpriseAuthAppId = dto.getUnionEnterpriseAuthAppId();

        List<TabSysUnionEnterpriseAuthApp> appList = unionEnterpriseAuthAppService.listUnionEnterpriseAuthApp(dto.getUnionEnterpriseAuthId());
        Set<String> appIdSet = new HashSet<>();
        if (CollectionUtils.isNotEmpty(appList)) {
             appList.forEach(e -> {
                 //编辑的时候去除本条数据的应用ID，避免校验错误
                 if (!e.getUnionEnterpriseAuthAppId().equals(unionEnterpriseAuthAppId)) {
                     String[] appIdArr = e.getAppId().split(",");
                     for (String appId : appIdArr) {
                         if (StringUtils.isBlank(appId)) {
                             continue;
                         }
                         appIdSet.add(appId);
                     }
                 }
            });
        }

        String appIdParam = dto.getAppId();
        String[] appIdParamArr = appIdParam.split(",");
        for (String temp : appIdParamArr) {
            if (StringUtils.isBlank(temp)) {
                continue;
            }
            if (appIdSet.contains(temp)) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "应用已经配置策略");
            }
        }

        if (unionEnterpriseAuthAppService.isRepeatStrategyName(unionEnterpriseAuthAppId, dto.getUnionEnterpriseAuthId(), dto.getStrategyName())) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "策略名称重复");
        }

        if (unionEnterpriseAuthAppId == null) {
            unionEnterpriseAuthAppService.saveUnionEnterpriseAuthApp(dto);
        } else {
            unionEnterpriseAuthAppService.editUnionEnterpriseAuthApp(dto);
        }

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UnionEnterpriseAuthAppDTO> getUnionEnterpriseAuthApp(@NotNull Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = checkUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        return ServiceResponse.success(EntityUtil.changeEntityNew(UnionEnterpriseAuthAppDTO.class, old));
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthAppDTO>> listUnionEnterpriseAuthApp(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        List<TabSysUnionEnterpriseAuthApp> list = unionEnterpriseAuthAppService.listUnionEnterpriseAuthApp(old.getUnionEnterpriseAuthId());
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UnionEnterpriseAuthAppDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<String> deleteUnionEnterpriseAuthApp(@NotNull Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = checkUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        unionEnterpriseAuthAppService.deleteUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        return ServiceResponse.success(old.getStrategyName());
    }

    @Override
    public ServiceResponse<String> deleteUnionEnterpriseAuthAppByUnionEnterpriseAuthId(@NotNull Integer unionEnterpriseAuthId) {
        unionEnterpriseAuthAppService.deleteUnionEnterpriseAuthAppByUnionEnterpriseAuthId(unionEnterpriseAuthId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveTask(String param) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listAll();
        if (CollectionUtils.isNotEmpty(list)) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (TabSysUnionEnterpriseAuth temp : list) {
                if (temp.getRelieveTime() != null && temp.getRelieveEffectDate() != null) {
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(temp.getRelieveTime());
                    calendar.add(Calendar.DAY_OF_YEAR, temp.getRelieveEffectDate());
                    //如果到达解除时间，发通知
                    if (sdf.format(calendar.getTime()).compareTo(sdf.format(new Date())) >= 0) {
                        if (UnionEnterpriseAuthStatusEnum.TO_DO_RELIEVE.getCode().equals(temp.getStatusFlag())) {
                            //todo 发通知
                            //门店解除
                            storeAuthorizationApiService.cancelStoreAll(temp.getUnionEnterpriseId(), temp.getOwnEnterpriseId());
                        }
                    }
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<OwnEnterpriseAuthListDTO>> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId, String startTime, String endTime) {
        List<OwnEnterpriseAuthListDTO> list = unionEnterpriseAuthService.listOwnEnterpriseAuth(unionEnterpriseId, startTime, endTime);
        if (CollectionUtils.isNotEmpty(list)) {
            //组装数据
            //1:自有商户数据
            List<Integer> enterpriseIdList = list.stream().mapToInt(e -> e.getEnterpriseId()).boxed().collect(Collectors.toList());
            Map<Integer, TabEnterprise> enterpriseMap = getEnterprise(enterpriseIdList);
            //2:超管数据
            Map<Integer, UserDTO> userMap = getUser(enterpriseIdList);
            //填充数据
            for (OwnEnterpriseAuthListDTO obj : list) {
                TabEnterprise enterprise = enterpriseMap.get(obj.getEnterpriseId());
                if (enterprise != null) {
                    obj.setCompanyName(enterprise.getCompanyName());
                    obj.setAddress(enterprise.getAddress());
                    obj.setEnterpriseName(enterprise.getEnterpriseName());
                    obj.setLogo(enterprise.getLogo());
                }
                UserDTO user = userMap.get(obj.getEnterpriseId());
                if (user != null) {
                    obj.setUserName(user.getUserName());
                    obj.setPhoneAreaCode(user.getPhoneAreaCode());
                    obj.setPhoneNumber(user.getPhoneNumber());
                }
            }
            return ServiceResponse.success(list);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<Page<UnionEnterpriseAuthResLogDTO>> listUnionEnterpriseAuthResLog(UnionEnterpriseAuthResLogQO param) {
        com.github.pagehelper.Page page = unionEnterpriseAuthResLogService.listUnionEnterpriseAuthResLog(param);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, UnionEnterpriseAuthResLogDTO.class));
    }

    private Map<Integer, TabEnterprise> getEnterprise(List<Integer> enterpriseIdList) {
        List<TabEnterprise> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIdList);
        if (CollectionUtils.isNotEmpty(enterpriseList)) {
            return enterpriseList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId(), e -> e));
        }
        return new HashMap<>();
    }

    private Map<Integer, UserDTO> getUser(List<Integer> enterpriseIdList) {
        ServiceResponse<List<UserDTO>> userResponse = userApiService.listUserByEnterpriseId(enterpriseIdList);
        if (userResponse.isSuccess()) {
            List<UserDTO> userList = userResponse.getResult();
            if (CollectionUtils.isNotEmpty(userList)) {
                return userList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId(), e -> e));
            }
        }
        return new HashMap<>();
    }

    private TabSysUnionEnterpriseAuth checkExistUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户数据不存在");
        }
        return old;
    }

    private TabSysUnionEnterpriseAuthApp checkUnionEnterpriseAuthApp(Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = unionEnterpriseAuthAppService.getUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "策略不存在");
        }
        return old;
    }

    private void checkResourceDealing(Integer unionEnterpriseAuthId) {
        List<TabSysUnionEnterpriseAuthResDetail> list = unionEnterpriseAuthResDetailService.listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            if (list.stream().anyMatch(e -> UnionEnterpriseAuthDetailStatusEnum.RELIEVING.getCode().equals(e.getAuthStatus())
                    || UnionEnterpriseAuthDetailStatusEnum.RESOURCE_DEALING.getCode().equals(e.getAuthStatus()))) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "当前存在“资源处理中”或“资源解除中”，不允许同步");
            }
        }
    }

}
