package com.gic.enterprise.service.outer.impl.union;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResOperateEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResTypeEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthStatusEnum;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthResDetail;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.UnionEnterpriseAuthResDetailApiService;
import com.gic.enterprise.service.UnionEnterpriseAuthResDetailService;
import com.gic.enterprise.service.UnionEnterpriseAuthResLogService;
import com.gic.enterprise.service.UnionEnterpriseAuthService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 联合资源回调接口
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/8 10:27 AM 
 */
@Service("unionEnterpriseAuthResDetailApiService")
public class UnionEnterpriseAuthResDetailApiServiceImpl implements UnionEnterpriseAuthResDetailApiService{
    @Autowired
    private UnionEnterpriseAuthResDetailService unionEnterpriseAuthResDetailService;
    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;
    @Autowired
    private UnionEnterpriseAuthResLogService unionEnterpriseAuthResLogService;

    @Override
    public ServiceResponse<Void> authDataCallback() {
        return null;
    }

    @Override
    public ServiceResponse<Void> unionForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId,
                                               String authKey, String remark) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode());

        boolean isUnionSuccess = UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode().equals(authStatus);
        //日志
        if (isUnionSuccess) {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(),
                    UnionEnterpriseAuthResOperateEnum.UNION.getMessage(), null, new Date());
        } else {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(),
                    UnionEnterpriseAuthResOperateEnum.UNION.getMessage(), new Date(), null);
        }
        //如果是，则判断其他资源是否已经完成
        if (isUnionSuccess && isAllDealSuccess(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(), true)) {
            //更新最新联合时间（）
            UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
            dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
            dto.setLastUnionTime(new Date());
            unionEnterpriseAuthService.update(dto);
        }
        unionEnterpriseAuthResDetailService.union(detail.getUnionEnterpriseAuthResDetailId(), authStatus, authKey, remark);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode());
        boolean isAllRelieveSuccess = UnionEnterpriseAuthDetailStatusEnum.HAS_RELIEVE.getCode().equals(authStatus);
        if (isAllRelieveSuccess) {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(),
                    UnionEnterpriseAuthResOperateEnum.RELIEVED.getMessage(), null, new Date());
        } else {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(),
                    UnionEnterpriseAuthResOperateEnum.RELIEVED.getMessage(), new Date(), null);
        }
        //如果是，则判断其他资源是否已经完成
        UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
        dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());

        if (isAllRelieveSuccess && isAllDealSuccess(old.getUnionEnterpriseAuthId(), UnionEnterpriseAuthResTypeEnum.STORE.getCode(), false)) {
            //更新最新联合时间（）
            dto.setLastRelieveTime(new Date());
            dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.RELIEVED.getCode());
        } else {
            dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.RELIEVING.getCode());
        }
        unionEnterpriseAuthService.update(dto);
        unionEnterpriseAuthResDetailService.union(detail.getUnionEnterpriseAuthResDetailId(), authStatus, null, null);
        return ServiceResponse.success();
    }

    private void unionCommon(Integer authStatus, Integer ownEnterpriseId, Integer unionEnterpriseId, String authKey, String remark) {

    }

    /**
     * 其他资源是否授权成功/解除成功
     * @param unionEnterpriseAuthId
     * @param resourceType
     * @param isUnion
     * @return
     */
    private boolean isAllDealSuccess(Integer unionEnterpriseAuthId, Integer resourceType, boolean isUnion) {
        Integer authStatus = UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode();
        if (!isUnion) {
            authStatus = UnionEnterpriseAuthDetailStatusEnum.HAS_RELIEVE.getCode();
        }
        boolean isAllDealSuccess = true;
        List<TabSysUnionEnterpriseAuthResDetail> resList = unionEnterpriseAuthResDetailService.listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(resList)) {
            for (TabSysUnionEnterpriseAuthResDetail temp : resList) {
                if (!temp.getResourceType().equals(resourceType) && !temp.getAuthStatus().equals(authStatus)) {
                    isAllDealSuccess = false;
                }
            }
        }
        return isAllDealSuccess;
    }

}
